//---------------------------------------------------------------------------
//
// Name: wind.cpp
// Autor: Fabian Braun - EL6A     
// Firma: Setila AG Widnau
// Datum: April - Juni 2004
// Beschreibung: Testprogramm fr Grafikdisplay mit Toshiba T6963 Controller
// Anschlussbelegung LPT-Port:
//
// 25-pol. Stecker   -->   Display
// -------------------------------
// 1 (Strobe)        -->   /WR
// 2..9 (D0..D7)     -->   D0..D7
// 14 (Autofeed)     -->   /CE
// 16 (Init)         -->   C/D
// 17 (Select in)    -->   /RES
// 18..25 (GND)      -->   GND
//
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include "wind.h"
#include "inpout32.c"         //DLL-Routinen einbinden

#define  TXT_HOME    0x0000   //Startadresse (RAM) des Textbereichs
#define  TXT_AREA    0x0028   //Lnge des Textbereichs pro Zeile
#define  GRAPH_HOME  0x0140   //Startadresse (RAM) des Grafikbereichs
#define  GRAPH_AREA  0x0028   //Lnge des Grafikbereichs pro Zeile

#define  LCD_PIXELX  240      //Anzahl Pixel - Breite
#define  LCD_PIXELY  64       //Anzahl Pixel - Hhe
#define  LCD_TEXTX   40       //Anzahl Zeichen pro Zeile
#define  LCD_TEXTY   8        //Anzahl Zeilen
#define  LCD_FONTWIDTH  6     //Zeichenbreite

#define  P_ADDR   0x0378      //IO-Adresse des Druckerports

//Displaysteuereingnge
#define  _WR   0              //Active Low
#define  _RE   3              //Active Low
#define  _CE   1              //Active Low
#define  CD    2              //Active High

//Prototyping
//------------
void glcd_init_io(void);
void glcd_init(void);
void glcd_write_data(short data);
short glcd_read_data(void);
void glcd_command(short cmd);
short glcd_status(void);
void glcd_busy(void);
void glcd_char (short);
void glcd_cls(short address_start, short address_end);
void glcd_position(short x, short y);

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFmain *Fmain;
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TFmain::TFmain(TComponent* Owner)
   : TForm(Owner)
{}

// Wird bei Programmstart aufgerufen
//---------------------------------------------------------------------------
void __fastcall TFmain::FormCreate(TObject *Sender)
{
   //Lnge des Textfelds begrenzen - Schlechte Variante wegen Zeilenumbrche nicht genau
   TFtext->MaxLength = LCD_TEXTX * LCD_TEXTY;
   //Standarttext ausgeben - Copyright ;-)
   TFtext->Text = "###################\r\nGLCD Testprogramm\r\nToshiba T6963C Controller\r\n\r\nFabian Braun\r\nSetila AG Widnau\r\nSommer 2004\r\n###################";

   //DLL laden
   if (dll(1) == 1)
   {
        //erfolgreich
        //MessageDlg("DLL zur LPT-Adressierung erfolgreich geladen!", mtInformation, TMsgDlgButtons() << mbOK, 0);
   }
   else
   {
        //fehlgeschlagen
        MessageDlg("Fehler beim laden der DLL \"inpout32.dll\"!", mtInformation, TMsgDlgButtons() << mbOK, 0);
        //Programm beenden
        exit(EXIT_FAILURE);
   }

   //IO initialisieren
   glcd_init_io();
   //LCD initialisieren
   glcd_init();
}

// Wird bei Schliessung des Fensters aufgerufen
//---------------------------------------------------------------------------
void __fastcall TFmain::FormDestroy(TObject *Sender)
{
   //DLL entladen
   dll(0) == 1;
}

// Ganzer Text aus dem Textfeld an das Display bertragen
//---------------------------------------------------------------------------
void __fastcall TFmain::BtextClick(TObject *Sender)
{
   short pos_x=0, pos_y=0;                         //Aktuelle X/Y Position des Adresspointers
   char *ptr = TFtext->Text.c_str();               //Char Pointer auf AnsiString setzen (AnsiString -> char *)

   //Textbereich des Displays lschen
   glcd_cls(TXT_HOME, LCD_TEXTX*LCD_TEXTY+TXT_HOME);
   //An den Anfang des Textbereichs setzen
   glcd_position(pos_x,pos_y);

   //Solange Display noch nicht voll und Text noch nicht leer ist Daten senden
   while (*ptr != 0x00 && !(pos_y == LCD_TEXTY && pos_x < LCD_TEXTX))
   {
      if((*ptr == '\r') && (*(ptr+1) == '\n'))
      //Als nchstes kommt ein Zeilenumbruch
      {
         pos_y++;                                  //nchste Zeile
         pos_x = 0;                                //Zeilenanfang
         glcd_position(pos_x, pos_y);              //Position wechseln
         ptr += 2;                                 //Nchste zwei Zeichen berspringen
      }
      else
      {
         glcd_char(*(ptr++));                      //Kein Zeilenumbruch -> Zeichen ausgeben
      }

      if (pos_x < (LCD_TEXTX-1))
      //Cursor noch nicht am Zeilenende
      {
         pos_x++;                                  //Nchste Position auf Zeile
      }
      else
      {
      //Cursor am Zeilenende
         pos_y++;                                  //Nchste Zeile
         pos_x = 0;                                //Zeilenanfang
      }
   }
}

// Text- und Grafikberech lschen
//---------------------------------------------------------------------------
void __fastcall TFmain::BclsClick(TObject *Sender)
{
   //Textbereich lschen
   glcd_cls(TXT_HOME, TXT_AREA*LCD_TEXTY);
   //Grafikbereich lschen
   glcd_cls(GRAPH_HOME, GRAPH_HOME+GRAPH_AREA*LCD_PIXELY);
}

// Dialog aufrufen und gewhltes Bild (JPEG oder Bitmap) laden
//---------------------------------------------------------------------------
void __fastcall TFmain::BimgloadClick(TObject *Sender)
{
   if (Dpic->Execute())                                     //Dialog aufrufen
   //Es wurde etwas ausgewhlt --> auswerten
   {
      AnsiString FName = Dpic->FileName;                    //Dateiname (AnsiString)
      char *FNameC = FName.c_str();                         //Dateiname (C-String)
      char *FName_cmp0 = {".jpg"};                          //1. Vergleichsstring
      char *FName_cmp1 = {".jpeg"};                         //2. Vergleichsstring

      if (FileExists(FName))                                //Datei auf Existenz berprfen
      {
         TJPEGImage *jpg = new TJPEGImage;                  //JPG-Objekt erstellen


         if (strstr(FNameC, FName_cmp0) || strstr(FNameC, FName_cmp1))
         //JPEG Datei
         {
            //"Konvertierung" von JPEG zu Bitmap
            jpg->LoadFromFile(FName);                       //JPG-Objekt mit Datei laden
            Pimg->Picture->Bitmap->Assign(jpg);             //Bitmap mit Bilddaten des JPG-Objekts laden
         }
         else
         //BMP Datei
         {
            Pimg->Picture->LoadFromFile(FName);             //Bitmap mit Datei laden
         }
      }
   }
}

// Daten an das Display senden
//---------------------------------------------------------------------------
void glcd_data(short dat)
{
   outp(P_ADDR, (dat & 0xFF));                  //Daten ausgeben
   clear_bit(P_ADDR+2, CD);                     //Daten senden
   set_bit(P_ADDR+2, _WR);                      //Schreiben aktivieren
   set_bit(P_ADDR+2, _CE);                      //Chip anwhlen

   clear_bit(P_ADDR+2, _CE);                    //Chip abwhlen
   clear_bit(P_ADDR+2, _WR);                    //Schreiben deaktivieren
}

// Befehl an das Display senden
//---------------------------------------------------------------------------
void glcd_command(short cmd)
{
   outp(P_ADDR, (cmd & 0xFF));                  //Daten ausgeben

   set_bit(P_ADDR+2, CD);                       //Daten senden
   set_bit(P_ADDR+2, _WR);                      //Schreiben aktivieren
   set_bit(P_ADDR+2, _CE);                      //Chip anwhlen

   clear_bit(P_ADDR+2, _CE);                    //Chip abwhlen
   clear_bit(P_ADDR+2, _WR);                    //Schreiben deaktivieren
}

// ASCII-Zeichen auf dem Display ausgeben
//---------------------------------------------------------------------------
void glcd_char(short data)
{
   glcd_data(data - 0x20);                      //Offset reduzieren von ASCII-Code
   glcd_command(0xC0);                          //Befehl Schreiben und inkrementieren
}

// C-String auf dem Display ausgeben
//---------------------------------------------------------------------------
void glcd_string(short *data)
{
   while(*data != 0x00)                         //Solange String nicht fertig
   {
      glcd_char(*(data++));                     //Aktuelles Zeichen senden und Pointer inkrementieren
   }
}

// Gewnschter Bereich des Displays bzw. RAMs lschen
//---------------------------------------------------------------------------
void glcd_cls(short address_start, short address_end)
{
   int i;

   //Address Pointer setzen
   glcd_data(address_start & 0x00FF);           //Low-Byte
   glcd_data((address_start & 0xFF00) >> 8);    //High-Byte
   glcd_command(0x24);                          //"Address Pointer Set"-Command

   //Den gewnschen Bereich lschen
   for (i = 0; i < (address_end-address_start); i++)
   {
      glcd_char(0x20);                          //Leerzeichen senden
   }
}

// Ein- und Ausgnge des Druckerports initialisieren
//---------------------------------------------------------------------------
void glcd_init_io(void)
{
   //Signale initialisieren
   clear_bit(P_ADDR+2, _WR);
   clear_bit(P_ADDR+2, _CE);
   set_bit(P_ADDR+2, CD);

   //Hardwarereset 
   set_bit(P_ADDR+2, _RE);
   Sleep(50);
   clear_bit(P_ADDR+2, _RE);

   Sleep(10);
}

// Display initialisieren (RAM Aufteilung, Mode Set, Display Mode Set, ...)
//---------------------------------------------------------------------------
void glcd_init(void)
{
  //Text Home Address setzen
   glcd_data(TXT_HOME & 0x00FF);
   glcd_data((TXT_HOME & 0xFF00) >> 8);
   glcd_command(0x40);
   //Text Area setzen
   glcd_data(TXT_AREA & 0x00FF);
   glcd_data((TXT_AREA & 0xFF00) >> 8);
   glcd_command(0x41);

   //Graphic Home Address setzen
   glcd_data(GRAPH_HOME & 0x00FF);
   glcd_data((GRAPH_HOME & 0xFF00) >> 8);
   glcd_command(0x42);
   //Graphic Area setzen
   glcd_data(GRAPH_AREA & 0x00FF);
   glcd_data((GRAPH_AREA & 0xFF00) >> 8);
   glcd_command(0x43);

   //Mode Set - je nach Status der Radio-Buttons
   glcd_command(0x80 | (Fmain->RBand->Checked * 0x03) | (Fmain->RBxor->Checked & 0x01));
   //Display Mode Set - je nach Status der Checkboxes
   glcd_command(0x93 | (Fmain->Ctext->Checked * 0x04) | (Fmain->Cgrafik->Checked * 0x08));

   //Addresspointer "homen"
   glcd_position(0, 0);
}


// Position des Addresspointers und des Cursors setzen

//---------------------------------------------------------------------------

void glcd_position(short x, short y)

{

   //RAM Adresse berechnen

   short address = (LCD_TEXTX * y) + x;


   //Cursor zur gewnschten Position wechseln

   glcd_data(address & 0x00FF);

   glcd_data((address & 0xFF00) >> 8);

   glcd_command(0x21);


   //Addresspointer zur gewnschten Position wechseln

   glcd_data(address & 0x00FF);

   glcd_data((address & 0xFF00) >> 8);

   glcd_command(0x24);

}

// Grafikebene ein- oder ausblenden
//---------------------------------------------------------------------------
void __fastcall TFmain::CgrafikClick(TObject *Sender)
{
   glcd_init();                                 //Display neu initialisieren
}

// Textebene ein- oder ausblenden
//---------------------------------------------------------------------------
void __fastcall TFmain::CtextClick(TObject *Sender)
{
   glcd_init();                                 //Display neu initialisieren
}

// AND-Modus
//---------------------------------------------------------------------------
void __fastcall TFmain::RBandClick(TObject *Sender)
{
   glcd_init();                                 //Display neu initialisieren
}

// OR-Modus
//---------------------------------------------------------------------------
void __fastcall TFmain::RBorClick(TObject *Sender)
{
   glcd_init();                                 //Display neu initialisieren
}

// XOR-Modus
//---------------------------------------------------------------------------
void __fastcall TFmain::RBxorClick(TObject *Sender)
{
   glcd_init();                                 //Display neu initialisieren
}

// Bilddaten aus dem "Image-Container" in den Grafikbereich des Displays schreiben
//---------------------------------------------------------------------------
void __fastcall TFmain::BimgClick(TObject *Sender)
{
   int x,y,z;                                   //Schleifenvariablen
   TColor p;                                    //Zwischenspeicher der Farbe
   short i;                                     //Displaydaten
   float pp;                                    //Farbberechung

   glcd_init();                                 //Display neu initialisieren

   //Grafikbereich lschen
   glcd_cls(GRAPH_HOME, GRAPH_HOME+GRAPH_AREA*LCD_PIXELY);

   //Addresspointer auf Grafikbereich setzen
   glcd_data(GRAPH_HOME & 0x00FF);
   glcd_data((GRAPH_HOME & 0xFF00) >> 8);

   glcd_command(0x24);

   for (y=0;y<LCD_PIXELY;y++)                   //Zeilenschleife
   {
      for (x=0;x<(LCD_PIXELX/LCD_FONTWIDTH);x++)//Spaltenschleife
      {
         i=0;                                   //Displaydaten lschen
         for(z=0;z<LCD_FONTWIDTH;z++)           //Ein "Displaybyte" berechnen
         {
            //Entsprechendes Pixel anwhlen und zwischenspeichern
            p=Pimg->Picture->Bitmap->Canvas->Pixels[x*LCD_FONTWIDTH+z][y];
            //Farbwert (0-255) aus Farbdaten des Pixels berechnen, wobei
            //die jeweiligen Grundfarben anders ins Gewicht fallen
            pp=0.6*(p & 0xFF)+0.3*((p >> 8) & 0xFF)+0.1*((p >> 16) & 0xFF);
            i=i << 1;                           //Nchstes Byte bearbeiten
            if (CInvers->Checked) pp=256-pp;    //Invertiert? Ja -> pp Invertieren
            //Wenn Farbwert grsser als Kontrast -> Pixel schwarz, sonst Pixel weiss
            if (pp < SBcontrast->Position) i=i | 1;
         }
         glcd_data(i);                          //"Displaybyte" senden
         glcd_command(0xC0);                    //Daten bernehmen und Adresspointer inkrementieren
      }
   }
}

//---------------------------------------------------------------------------
// END OF FILE
//---------------------------------------------------------------------------


